---------------------------------------------------------------------------
-- DEMO : FTS et FileTable
-- Romans pdf
-- Dmo DATAFROGS 2023
-- https://blog.datafly.pro
---------------------------------------------------------------------------
USE [master]
GO
---------------------------------------------------------------------------
-- Installation
---------------------------------------------------------------------------
-- Quels sont les filtres disponibles
EXEC sp_help_fulltext_system_components 'filter';   

-- Installation Adobe pdf iFilter 9
-- Mettre le dossier du ifilter dans le path systme
-- puis :
EXEC sp_fulltext_service @action='load_os_resources', @value=1;
EXEC sp_fulltext_service @action='verify_signature', @value=0;

-- Vrification
SELECT * FROM sys.fulltext_document_types WHERE document_type IN ('.docx' ,'.pdf')
-- Redmarrage fdhost
EXEC sp_fulltext_service 'restart_all_fdhosts';

-- Cration de la base
CREATE DATABASE [Books]
 CONTAINMENT = NONE
 ON  PRIMARY 
( NAME = N'Books', FILENAME = N'E:\MSSQLData\MSSQL16.SQL2022\MSSQL\Data\Books.mdf' , SIZE = 8192KB , MAXSIZE = UNLIMITED, FILEGROWTH = 65536KB ),
 FILEGROUP [BooksFT] CONTAINS FILESTREAM 
( NAME = N'[BooksFT]', FILENAME = N'F:\SQL_BooksFT\' )
 LOG ON 
( NAME = N'Books_log', FILENAME = N'E:\MSSQLData\MSSQL16.SQL2022\MSSQL\Data\Books_log.ldf' , SIZE = 8192KB , MAXSIZE = 2048GB , FILEGROWTH = 65536KB )
GO
ALTER DATABASE [Books]
SET FILESTREAM ( NON_TRANSACTED_ACCESS = FULL, DIRECTORY_NAME = N'Books' )
GO

-- Cration de la table
USE Books
GO
DROP TABLE IF EXISTS BooksStore
CREATE TABLE BooksStore AS FILETABLE
WITH 
( 
   FILETABLE_DIRECTORY = 'BooksStore'
 , FILETABLE_COLLATE_FILENAME = database_default
 , FILETABLE_PRIMARY_KEY_CONSTRAINT_NAME = PK_BookStore  --hierarchyid
 , FILETABLE_STREAMID_UNIQUE_CONSTRAINT_NAME = UQ_stream_id --uniqueidentifier
)
GO

-- Cration du catalogue
ALTER DATABASE Books
   ADD FILEGROUP FG_Books_FT;
GO
ALTER DATABASE [Books]
ADD FILE
    (
        NAME = N'Books_FT'
      , FILENAME = N'E:\MSSQLData\MSSQL16.SQL2022\MSSQL\Data\Books_FT.NDF'
    )
TO FILEGROUP [FG_Books_FT];
GO
CREATE FULLTEXT CATALOG FTC_Books
   ON FILEGROUP FG_Books_FT
     WITH ACCENT_SENSITIVITY = OFF
   AS DEFAULT;
GO

-- Cration de l'index
CREATE FULLTEXT INDEX
   ON dbo.BooksStore (file_stream TYPE COLUMN file_type LANGUAGE 1036) 
   KEY INDEX UQ_stream_id
       ON (FTC_Books, FILEGROUP FG_Books_FT)
   WITH  (CHANGE_TRACKING = AUTO, 
          STOPLIST        = SYSTEM);

-- Charger quelques livres
-- ...
SELECT * FROM sys.dm_fts_index_population WHERE database_id = DB_ID()

---------------------------------------------------------------------------
-- Queries
---------------------------------------------------------------------------

SELECT TOP 10
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath(1) AS FILEPATH
FROM BooksStore
WHERE CONTAINS(file_stream,'"Big Brother"')

SELECT TOP 10
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FILEPATH
FROM BooksStore
WHERE CONTAINS(file_stream,'"Arsne" AND "Lupin"')

SELECT TOP 10
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FILEPATH
FROM BooksStore
WHERE CONTAINS(file_stream,'"Holmes" AND "Watson"')

SELECT TOP 10
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FILEPATH
FROM BooksStore
WHERE CONTAINS(file_stream,'"sous-marin" AND "narval"')

---------------------------------------------------------------------------
-- DMV
---------------------------------------------------------------------------
-- Mots indexs (approximatif)
SELECT * FROM sys.dm_fts_index_keywords(DB_ID(),OBJECT_ID('BooksStore'))
ORDER BY document_count DESC

-- Documents contenant un mot
SELECT 
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FilePath
FROM BooksStore
WHERE CONTAINS(*,'homme')

-- Mots indexs par document : document_id
SELECT * FROM sys.dm_fts_index_keywords_by_document(DB_ID(),OBJECT_ID('BooksStore'))
WHERE display_term = 'homme'

-- Mapper les id de documents sur leur cl
DROP TABLE IF EXISTS #Mapping
CREATE TABLE #Mapping (docid INT PRIMARY KEY, [key]	UNIQUEIDENTIFIER NOT NULL)
DECLARE @table_id INT = OBJECT_ID('BooksStore')
EXEC sp_fulltext_keymappings @table_id
INSERT #Mapping EXEC sp_fulltext_keymappings @table_id

-- Rechercher les documents (et les trouver avec leur cl)
SELECT KBD.display_term,KBD.occurrence_count,BS.name 
FROM sys.dm_fts_index_keywords_by_document(DB_ID(),OBJECT_ID('BooksStore')) KBD
JOIN #Mapping M ON KBD.document_id = M.docid
JOIN dbo.BooksStore AS BS ON M.[key] = BS.stream_id
WHERE display_term = 'homme'
ORDER BY KBD.occurrence_count DESC

-- Position d'un mot dans le document
SELECT KPD.*,file_stream.GetFileNamespacePath() 
FROM sys.dm_fts_index_keywords_position_by_document(DB_ID(),OBJECT_ID('BooksStore')) KPD
JOIN #Mapping M ON KPD.document_id = M.docid
JOIN dbo.BooksStore AS BS ON M.[key] = BS.stream_id
WHERE display_term = 'Watson'

---------------------------------------------------------------------------
--Recherche dans les proprits de documents
---------------------------------------------------------------------------
CREATE SEARCH PROPERTY LIST [Books_SearchPropertyList] ;

ALTER SEARCH PROPERTY LIST Books_SearchPropertyList  
   ADD 'Title'  
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 2,   
      PROPERTY_DESCRIPTION = 'System.Title - Title of the item.' );  

ALTER SEARCH PROPERTY LIST Books_SearchPropertyList  
   ADD 'Author'  
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 4,   
      PROPERTY_DESCRIPTION = 'System.Author - Author of the item.' );  

ALTER FULLTEXT INDEX ON dbo.BooksStore SET SEARCH PROPERTY LIST Books_SearchPropertyList;

-- Mots indexs dans les proprits
SELECT * FROM sys.dm_fts_index_keywords_by_property  
(   
DB_ID(),   
OBJECT_ID('BooksStore')   
)  
ORDER BY document_id, property_id,keyword

-- Recherche dans les proprits
SELECT TOP (10)
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FilePath
FROM BooksStore
WHERE CONTAINS(PROPERTY(file_stream, 'Title'), '"Lupin" AND "Cambrioleur"'); 

SELECT TOP 10
       stream_id
     , name
     , file_type
     , cached_file_size
     , file_stream.GetFileNamespacePath() AS FilePath
FROM BooksStore
WHERE CONTAINS(PROPERTY(file_stream, 'Author'), '"Jules Verne"'); 

-- Highlight du terme recherch et affichage des proprits de documents 
-- Non pris en charge nativement !
-- Voir l'addin ThinkHighlight (CLR) :  https://www.interactivethoughts.com/products/thinkhighlight/



