---------------------------------------------------------------------------
-- DEMO : Recherche smantique 
-- Extraction d'expressions cls, comparaison de documents
-- Dmo DATAFROGS 2023
-- https://blog.datafly.pro
---------------------------------------------------------------------------

---------------------------------------------------------------------------
-- Installation
---------------------------------------------------------------------------
-- Attacher la base semanticdb
CREATE DATABASE semanticdb ... FOR ATTACH 
-- Inscrire la base
EXEC sp_fulltext_semantic_register_language_statistics_db @dbname = N'semanticsdb'
-- Vrifier les langues prises en charge
SELECT * FROM sys.fulltext_semantic_languages  

USE WikipediaSemantic
GO
-- Cration du catalogue full-text
CREATE FULLTEXT CATALOG FTC_Wikipedia
   ON FILEGROUP FG_Wikipedia_FTC
     WITH ACCENT_SENSITIVITY = OFF
   AS DEFAULT;
-- Puis cration de l'index full-text
CREATE FULLTEXT INDEX
ON dbo.Articles (Titre LANGUAGE 1036 STATISTICAL_SEMANTICS, Article LANGUAGE 1036 STATISTICAL_SEMANTICS) 
KEY INDEX PK_Article
    ON (FTC_Wikipedia, FILEGROUP FG_Wikipedia_FTC)
WITH  (CHANGE_TRACKING = AUTO, 
        STOPLIST        = SYSTEM);
-- ou modification d'un index full-text dj existant
--ALTER FULLTEXT INDEX ON dbo.Articles ALTER COLUMN Article ADD STATISTICAL_SEMANTICS 

-- Surveiller la population de l'index
SELECT * FROM sys.dm_fts_index_population WHERE database_id = DB_ID()
SELECT * FROM sys.dm_fts_semantic_similarity_population WHERE database_id = DB_ID()

-- Configuration / performances
EXEC sp_configure 'max full-text crawl range',12  -- 4 par dfaut
RECONFIGURE

-- Taille
SELECT OBJECT_NAME(i.object_id) AS TableName, i.type_desc AS IndexType, SUM(a.total_pages * 8 / 1024) AS Size_Mo
FROM sys.tables t
     JOIN sys.indexes i ON t.object_id=i.object_id
     JOIN sys.partitions p ON i.object_id=p.object_id AND i.index_id=p.index_id
     JOIN sys.allocation_units a ON p.partition_id=a.container_id
WHERE t.type='U' AND i.index_id=1
GROUP BY OBJECT_NAME(i.object_id), i.type_desc
UNION
SELECT 'Articles', 'FullText', FULLTEXTCATALOGPROPERTY('FTC_Wikipedia', 'IndexSize');

SELECT * FROM sys.dm_db_fts_index_physical_stats

---------------------------------------------------------------------------
-- Queries
---------------------------------------------------------------------------

SELECT * FROM Articles WHERE ID = 1297567

-- mots cls dtects dans un document avec SEMANTICKEYPHRASETABLE
SELECT TOP(50) * 
FROM SEMANTICKEYPHRASETABLE(Articles, Article,1297567) AS SKPT
ORDER BY score DESC

-- recherche de documents qui contiennent une expression cl
SELECT TOP (25) * FROM Articles A INNER JOIN SEMANTICKEYPHRASETABLE(Articles,Article) AS SKPT
ON A.ID = SKPT.document_key
WHERE SKPT.keyphrase = 'poirot'
ORDER BY score DESC

-- recherche de documents similaires ou en relation avec SEMANTICSIMILARITYTABLE

INSERT Articles (Titre,Article) SELECT REPLACE(Titre,'Agatha Christie','Agatha Christie (copie)'),REPLACE(Article,'Torquay','Trifouillis-les-cactus') FROM Articles WHERE ID = 1297567
SELECT @@IDENTITY

DECLARE @SourceID INT = 2280963
SELECT @SourceID AS SourceID, (SELECT Titre FROM Articles WHERE ID = @SourceID) AS SourceTitre,
A.ID AS MatchedID, A.Titre AS MatchedTitre, SST.* 
FROM SEMANTICSIMILARITYTABLE
(
	Articles,
	Article,  -- (Titre,Article)
	@SourceID
) AS SST
INNER JOIN Articles A ON A.ID = SST.matched_document_key
ORDER BY score DESC
GO

-- recherche d'expressions cls communes avec SEMANTICSIMILARITYDETAILSTABLE

DECLARE @SourceID INT = 2280963
SELECT keyphrase,score 
FROM SEMANTICSIMILARITYDETAILSTABLE
(
	Articles,
	Article,1297567,
	Article,@SourceID
) 
ORDER BY score DESC


--DELETE FROM Articles WHERE ID > 2280949


